'use strict';
import '../src/angular-screenshot';

describe('screenshot directive', function () {
   const toolboxClass = '.screenshot-toolbox';
   let $compile, $rootScope;
   beforeEach(() => {
      window.module('angular-screenshot');
      inject(function (_$compile_, _$rootScope_) {
         $compile = _$compile_;
         $rootScope = _$rootScope_;
      });
   });

   const dragLeftTopToRightBottom = (canvasSelector) => {
      const offset = canvasSelector.offset();
      const startX = offset.left;
      const startY = offset.top;
      const endX = startX + (canvasSelector.width() / 2);
      const endY = startY + (canvasSelector.height() / 2);
      return dragSection(canvasSelector[0], startX, startY, endX, endY);
   };

   const dragRightBottomToLeftTop = (canvasSelector) => {
      const offset = canvasSelector.offset();
      const endX = offset.left;
      const endY = offset.top;
      const startX = endX + (canvasSelector.width() / 2);
      const startY = endY + (canvasSelector.height() / 2);
      return dragSection(canvasSelector[0], startX, startY, endX, endY);
   };

   const dragSection = (canvas, startX, startY, endX, endY) => {
      const mousedown = document.createEvent("MouseEvents");
      mousedown.initMouseEvent("mousedown", true, true, window, 0, 0, 0, startX, startY, false, false, false, false, 0, null);
      canvas.dispatchEvent(mousedown);
      const mousemove = document.createEvent('MouseEvents');
      mousemove.initMouseEvent("mousemove", true, true, window, 0, 0, 0, endX, endY, false, false, false, false, 0, null);
      canvas.dispatchEvent(mousemove);
      const mouseup = document.createEvent("MouseEvents");
      mouseup.initMouseEvent("mouseup", true, true, window, 0, 0, 0, endX, endY, false, false, false, false, 0, null);
      canvas.dispatchEvent(mouseup);
      return Promise.resolve({
         canvas,
         startX,
         startY,
         endX,
         endY
      });
   };

   const findMaxZindex = () => {
      let zMax = 0;
      angular.element('body *').each(function () {
         let zIndex = angular.element(this).css('zIndex');
         zIndex = parseInt(zIndex, 10);
         if (zIndex && zIndex > zMax) {
            zMax = zIndex;
         }
      });
      return zMax;
   };

   const getChildSelector = (element, childName) => element.find(childName);

   const waitFor = (timespan) => new Promise((resolve) => setTimeout(() => resolve(), timespan));

   describe('basic features', () => {
      let scope,
         absoluteElement,
         element,
         body,
         screenshotCtrl;
      beforeEach(() => {
         scope = $rootScope.$new();
         element = angular.element('<screenshot is-open="isOpen"><div>Hello World</div></screenshot>');
         absoluteElement = angular.element('<div></div>');
         $compile(element)(scope);
         scope.$digest();
         screenshotCtrl = element.isolateScope().screenshotCtrl;
         body = angular.element(document.body);
         body.width(500);
         body.height(500);
         body.append(absoluteElement);
         body.append(element);
         absoluteElement.css({ top: body.height() / 2, left: body.width() / 2, position: 'absolute', zIndex: 1 });
      });
      afterEach(() => {
         scope.isOpen = false;
         scope.$digest();
         body.find(element).remove();
         body.find(toolboxClass).remove();
         body.find(absoluteElement).remove();
      });
      it('does not rendered the canvas when isOpen = false', (done) => {
         //Arrange
         scope.isOpen = false;
         //Act
         scope.$digest();
         //Act/Assert
         waitFor().then(() => {
            const canvas = getChildSelector(body, 'canvas')[0];
            expect(canvas).toBeUndefined();
            expect(screenshotCtrl.isOpen).toEqual(scope.isOpen);
            done();
         });
      });

      it('should rendered the canvas when isOpen = true', (done) => {
         //Arrange
         scope.isOpen = true;
         //Act
         scope.$digest();
         //Act/Assert
         waitFor().then(() => {
            const canvas = getChildSelector(body, 'canvas')[0];
            expect(canvas).not.toBeUndefined();
            expect(screenshotCtrl.isOpen).toEqual(scope.isOpen);
            done();
         });
      });

      it('does not rendered the canvas when open then send right click on canvas', (done) => {
         //Arrange
         scope.isOpen = true;
         //Act/Assert
         scope.$digest();
         waitFor()
            .then(() => {
               let canvas = getChildSelector(body, 'canvas')[0];
               const events = document.createEvent('HTMLEvents');
               events.initEvent('contextmenu', true, false);
               canvas.dispatchEvent(events);
               canvas = getChildSelector(body, 'canvas')[0];
               expect(canvas).toBeUndefined();
               done();
            });
      });

      it('should rendered the toolbox when drag with section', (done) => {
         //Arrange
         scope.isOpen = true;
         //Act/Assert
         scope.$digest();
         waitFor()
            .then(() => {
               const canvasSelector = getChildSelector(body, 'canvas');
               return dragLeftTopToRightBottom(canvasSelector);
            })
            .then(() => {
               const toolboxSelector = getChildSelector(body, toolboxClass);
               expect(toolboxSelector.length).toBeGreaterThan(0);
               done();
            });
      });

      it('should rendered the toolbox when drag with section from right bottom to left top', (done) => {
         //Arrange
         scope.isOpen = true;
         //Act/Assert
         scope.$digest();
         waitFor()
            .then(() => {
               const canvasSelector = getChildSelector(body, 'canvas');
               return dragRightBottomToLeftTop(canvasSelector);
            })
            .then(() => {
               const canvasSelector = getChildSelector(body, 'canvas');
               const toolboxSelector = getChildSelector(body, toolboxClass);
               const canvasOffset = canvasSelector.offset();
               const toolboxOffset = toolboxSelector.offset();
               const toolboxHeight = toolboxSelector[0].clientHeight;
               expect(canvasOffset.left).toEqual(toolboxOffset.left);
               expect(canvasOffset.top).toEqual(Math.abs(toolboxOffset.top - toolboxHeight));
               done();
            });
      });

      it('should not rendered the toolbox when trigger the cancel', (done) => {
         //Arrange
         scope.isOpen = true;
         //Act/Assert
         scope.$digest();
         waitFor()
            .then(() => {
               const canvasSelector = getChildSelector(body, 'canvas');
               return dragLeftTopToRightBottom(canvasSelector);
            })
            .then(() => waitFor())
            .then(() => {
               screenshotCtrl.cancel();
               const toolboxSelector = getChildSelector(body, toolboxClass);
               expect(toolboxSelector.length).toEqual(0);
               done();
            });
      });

      it('should toolbox\'s z-index greater than canvas and origin max z-index', (done) => {
         //Arrange
         scope.isOpen = true;
         const originMaxZindex = findMaxZindex();
         //Act/Assert
         scope.$digest();
         waitFor()
            .then(() => {
               const canvasSelector = getChildSelector(body, 'canvas');
               return dragLeftTopToRightBottom(canvasSelector);
            })
            .then(() => {
               const canvasSelector = getChildSelector(body, 'canvas');
               const toolboxSelector = getChildSelector(body, toolboxClass);
               const canvasZindex = canvasSelector.css('z-index');
               const toolboxZindex = toolboxSelector.css('z-index');
               expect(toolboxZindex).toBeGreaterThan(canvasZindex);
               expect(canvasZindex).toBeGreaterThan(originMaxZindex);
               done();
            });
      });
   });
});
